from flask import Flask, render_template, request, redirect, url_for, flash, session, jsonify

from uuid import uuid4
from flask_socketio import SocketIO
import json

app = Flask(__name__)
app.config['SECRET_KEY'] = 'DEV'
socketio = SocketIO(app)

NAME_LENGTH = 50
MAX_PLAYERS = 100

def find_game_by_id(id:str) -> dict:
    games = []
    with open('games.json', 'r') as f:
        games = json.load(f)
    
    index = 0
    for game in games:
        if game['id'] == id:
            return {"game": game, "idx": index}
        index += 1
    return {"error": "Game not found"}

def start_game(game:dict):
    pass


@app.route('/new/<string:name>/<int:players>', methods=['POST'])
def new_game(name, players):
    # Log new game into json file and return a websocket url for the game
    if len(name) > NAME_LENGTH:
        return {"error": f"Game name too long (max {NAME_LENGTH} characters)"}
    
    # Check if too many players
    if players > MAX_PLAYERS:
        return {"error": f"Too many players (max {MAX_PLAYERS}))"}

    
    games = []
    with open('games.json', 'r') as f:
        games = json.load(f)
    
    # Create new game
    
    game = {
        "game": name,
        "players": players,
        "in": [request.remote_addr],
        "join": None,
        "id": f"{uuid4().hex}",
        "status": "waiting",
        "private":False if request.args.get('private') is None else request.args.get('private'),
    }
    games.append(game)

    with open('games.json', 'w') as f:
        json.dump(games, f)
    
    return jsonify(game)
    

@app.route('/games/')
def list_games(internal=False):
    # List all public games
    games = []
    with open('games.json', 'r') as f:
        games = json.load(f)
    games = [game for game in games if not game['private']]
    
    if internal:
        return games
    return jsonify(games)

@app.route('/games/<string:game_id>')
def get_game(game_id):
    # Get game by id
    game = find_game_by_id(game_id)
    if "error" in game:
        return game["error"]
    return jsonify(game['game'])

@app.route('/join/<uuid:game_id>', methods=['POST'])
def join_game(game_id):
    

    # Check if game exists
    game = find_game_by_id(game_id)
    if "error" in game:
        return game["error"]

    # Check if game is full
    game, idx = game['game'], game['idx']
    if len(game['in']) >= game['players']:
        return {"error": "Game is full"}

    # Add player to game
    game['in'].append(request.remote_addr)

    # Check if game is ready
    if len(game['in']) == game['players']:
        game['status'] = "ready"
        start_game(game)
        return {"status": "starting"}

    games = []
    with open('games.json', 'r') as f:
        games = json.load(f)
    games[game.index(game)] = game
    with open('games.json', 'w') as f:
        json.dump(games, f)


@app.route('/')
def index():
    # List games
    return render_template('index.html', games=list_games(internal=True))

if __name__ == '__main__':
    socketio.run(app, debug=True)
    

